const ko = require("knockout");
const _ = require("lodash");

const Dialog = require("cattura-ui-library/src/lib/ui/dialog");
const {apiv1} = require("./api");

const template = require("./content.pug");

const testQueryResults = require("../test-query-results");

const DEFAULT_FILTERS = {
    ANY:          "(objectClass=*)",
    ANY_USER:     "(objectClass=user)",
    ANY_ORG_UNIT: "(objectClass=organizationalUnit)"
};

function makeDialog (plugin)
{
    let roles = ko.observableArray();
    apiv1.users.getRoles().then(retval => _(retval).map((role, id) => _({
        id: id
    }).extend(role).value()).value()).then(roles);

    return Dialog({
        id: ko.observable(),
        roles: roles,

        fields: ko.validatedObservable({
            role: ko.observable(),
            dn: ko.observable().extend({
                required: true
            }),
            filter: ko.observable().extend({
                required: true
            }),
            userField: ko.observable(),
            passwordField: ko.observable()
        }),

        toQuery ()
        {
            let query = {
                role: this.fields().role(),
                dn: this.fields().dn(),
                filter: this.fields().filter(),
                userField: this.fields().userField(),
                passwordField: this.fields().passwordField()
            };

            if (this.id())
            {
                query.id = this.id();
            }

            return query;
        },

        open (query)
        {
            this.reset();

            if (query)
            {
                this.id(query.id);
                this.fields().role(query.role);
                this.fields().dn(query.dn);
                this.fields().filter(query.filter);
                this.fields().userField(query.userField);
                this.fields().passwordField(query.passwordField);
            }

            this.show();
        },

        reset ()
        {
            this.id(0);
            this.fields().role("");
            this.fields().dn("");
            this.fields().filter("");
            this.fields().userField("");
            this.fields().passwordField("");
        },

        save ()
        {
            if (!this.fields.isValid())
            {
                this.dispatchEvent({type: "error", message: "Form is not valid."});
                return;
            }

            const query = this.toQuery();
            let request;
            if (this.id() > 0)
            {
                request = plugin.services.updateQuery({
                    query: query
                });
            }
            else
            {
                request = plugin.services.addQuery({
                    query: query
                });
            }
            
            request.then(() => {
                this.dispatchEvent({type: "querysaved", query: query});
                this.hide();
            }).catch(error => this.dispatchEvent({type: "error", message: error.message, error: error}));
        },

        test ()
        {
            plugin.services.runQuery({
                query: this.toQuery()
            }).then(users => {
                testQueryResults.open(users);
                //Dialogs.wait.dismiss();
                //testQueryDialog(users);
            }).catch(error => {
                //Dialogs.wait.dismiss();
                this.dispatchEvent({type: "error", message: error.message, error: error})
            });
        }
    }, {
        template: template
    });
}

module.exports = makeDialog;
